/**
 ******************************************************************************
 * @file     ETH_Web_CAM.ino
 * @brief    Set up ESP32-S3 module with Ethernet (ETH) and camera functionality.
 *           This program initializes Ethernet connectivity and configures the
 *           camera for streaming or capturing images. The camera supports
 *           dynamic configuration based on PSRAM availability.
 * @version  V1.2
 * @date     2024-11-27
 * @license  MIT
 ******************************************************************************
 * Experiment Objective: Integrate Ethernet and camera functionality on ESP32-S3.
 *                       Enable camera data streaming over Ethernet.
 *
 * Hardware Resources:
 * 1. Ethernet Interface --> ESP32-S3 Module with external PHY.
 * 2. Camera Module --> Connected with specified pin mappings.
 *
 ******************************************************************************
 *
 * Development Platform: ESP32-S3 Development Board
 * Support Forum: service.waveshare.com
 * Company Website: www.waveshare.com
 *
 ******************************************************************************
 */

#include "esp_camera.h"
#include <WiFi.h>
#include <ETHClass.h>  // Modified ETHClass for Ethernet support
#include "utilities.h" // Board-specific pin mappings

// Define GPIO pins for the camera module
#define PWDN_GPIO_NUM   -1 // Power-down pin (not used)
#define RESET_GPIO_NUM  -1 // Reset pin (not used)
#define XCLK_GPIO_NUM   3  // XCLK pin
#define SIOD_GPIO_NUM   48 // SCCB SDA pin
#define SIOC_GPIO_NUM   47 // SCCB SCL pin
#define Y9_GPIO_NUM     18 // Y9 pin
#define Y8_GPIO_NUM     15 // Y8 pin
#define Y7_GPIO_NUM     38 // Y7 pin
#define Y6_GPIO_NUM     40 // Y6 pin
#define Y5_GPIO_NUM     42 // Y5 pin
#define Y4_GPIO_NUM     46 // Y4 pin
#define Y3_GPIO_NUM     45 // Y3 pin
#define Y2_GPIO_NUM     41 // Y2 pin
#define VSYNC_GPIO_NUM  1  // VSYNC pin
#define HREF_GPIO_NUM   2  // HREF pin
#define PCLK_GPIO_NUM   39 // PCLK pin

// Define infrared filter and LED control pins
#define IR_FILTER_NUM   -1
#define LED1            8

// Variables for network status
char macAddress[18] = {};
char localIP[16] = {};
char linkSpeed[10] = {};
char gatewayIP[16] = {};

static bool eth_connected = false; // Ethernet connection status

// Forward declarations
void startCameraServer();

/**
 ******************************************************************************
 * @brief    Event handler for WiFi and Ethernet events.
 * @param    event WiFi event type.
 ******************************************************************************
 */
void WiFiEvent(WiFiEvent_t event) {
    switch (event) {
        case ARDUINO_EVENT_ETH_START:
            printf("Ethernet Started\n");
            ETH.setHostname("esp32-ethernet");
            break;
        case ARDUINO_EVENT_ETH_CONNECTED:
            printf("Ethernet Connected\n");
            break;
        case ARDUINO_EVENT_ETH_GOT_IP:
            printf("Ethernet MAC: %s\n", ETH.macAddress().c_str());
            printf("IP Address: %s\n", ETH.localIP().toString().c_str());
            printf("Link Speed: %d Mbps\n", ETH.linkSpeed());
            printf("Gateway IP: %s\n", ETH.gatewayIP().toString().c_str());
            eth_connected = true;
            break;
        case ARDUINO_EVENT_ETH_DISCONNECTED:
            printf("Ethernet Disconnected\n");
            eth_connected = false;
            break;
        case ARDUINO_EVENT_ETH_STOP:
            printf("Ethernet Stopped\n");
            eth_connected = false;
            break;
        default:
            break;
    }
}

/**
 ******************************************************************************
 * @brief    Setup function to initialize Ethernet, camera, and peripherals.
 ******************************************************************************
 */
void setup() {
    Serial.begin(115200); // Initialize Serial communication
    Serial.setDebugOutput(true);

    WiFi.onEvent(WiFiEvent); // Register WiFi event handler
    pinMode(LED1, OUTPUT);   // Configure LED1 pin as output
    digitalWrite(LED1, LOW); // Ensure LED is off at startup

#ifdef ETH_POWER_PIN
    pinMode(ETH_POWER_PIN, OUTPUT);
    digitalWrite(ETH_POWER_PIN, HIGH);
#endif

    // Start Ethernet connection
#if CONFIG_IDF_TARGET_ESP32
    if (!ETH.begin(ETH_ADDR, ETH_RESET_PIN, ETH_MDC_PIN, ETH_MDIO_PIN, ETH_TYPE, ETH_CLK_MODE)) {
        printf("Ethernet initialization failed!\n");
    }
#else
    if (!ETH.beginSPI(ETH_MISO_PIN, ETH_MOSI_PIN, ETH_SCLK_PIN, ETH_CS_PIN, ETH_RST_PIN, ETH_INT_PIN)) {
        printf("Ethernet initialization failed!\n");
    }
#endif

    pinMode(IR_FILTER_NUM, OUTPUT); // Configure infrared filter

    // Wait for Ethernet connection
    while (!eth_connected) {
        printf("Waiting for Ethernet connection...\n");
        delay(1000);
    }

    // Configure camera
    camera_config_t config = {};
    config.ledc_channel = LEDC_CHANNEL_0;
    config.ledc_timer = LEDC_TIMER_0;
    config.pin_d0 = Y2_GPIO_NUM;
    config.pin_d1 = Y3_GPIO_NUM;
    config.pin_d2 = Y4_GPIO_NUM;
    config.pin_d3 = Y5_GPIO_NUM;
    config.pin_d4 = Y6_GPIO_NUM;
    config.pin_d5 = Y7_GPIO_NUM;
    config.pin_d6 = Y8_GPIO_NUM;
    config.pin_d7 = Y9_GPIO_NUM;
    config.pin_xclk = XCLK_GPIO_NUM;
    config.pin_pclk = PCLK_GPIO_NUM;
    config.pin_vsync = VSYNC_GPIO_NUM;
    config.pin_href = HREF_GPIO_NUM;
    config.pin_sccb_sda = SIOD_GPIO_NUM;
    config.pin_sccb_scl = SIOC_GPIO_NUM;
    config.pin_pwdn = PWDN_GPIO_NUM;
    config.pin_reset = RESET_GPIO_NUM;
    config.xclk_freq_hz = 20000000;
    config.pixel_format = PIXFORMAT_JPEG;
    config.frame_size = FRAMESIZE_UXGA;
    config.jpeg_quality = 12;
    config.fb_count = 1;

    // Adjust camera settings based on PSRAM availability
    if (psramFound()) {
        config.jpeg_quality = 10;
        config.fb_count = 2;
        config.grab_mode = CAMERA_GRAB_LATEST;
    } else {
        config.frame_size = FRAMESIZE_SVGA;
        config.fb_location = CAMERA_FB_IN_DRAM;
    }

    // Initialize camera
    esp_err_t err = esp_camera_init(&config);
    if (err != ESP_OK) {
        printf("Camera initialization failed! Error: 0x%x\n", err);
        return;
    }

    printf("Camera initialized successfully.\n");

    // Start the camera web server
    startCameraServer();
}

/**
 ******************************************************************************
 * @brief    Main loop, nothing to execute here as server runs independently.
 ******************************************************************************
 */
void loop() {
    delay(10000); // Idle delay to save CPU resources
}
